@@ -25,11 +25,17 @@ This module contains Base AWS Hook.
 from __future__ import annotations
 
 import datetime
+import inspect
 import json
 import logging
+import os
+import uuid
 import warnings
+from copy import deepcopy
 from functools import wraps
-from typing import Any, Callable, Generic, TypeVar, Union
+from os import PathLike
+from pathlib import Path
+from typing import TYPE_CHECKING, Any, Callable, Generic, TypeVar, Union
 
 import boto3
 import botocore
@@ -39,6 +45,7 @@ import tenacity
 from botocore.client import ClientMeta
 from botocore.config import Config
 from botocore.credentials import ReadOnlyCredentials
+from botocore.waiter import Waiter, WaiterModel
 from dateutil.tz import tzlocal
 from slugify import slugify
 
@@ -46,14 +53,20 @@ from airflow.compat.functools import cached_property
 from airflow.configuration import conf
 from airflow.exceptions import AirflowException, AirflowNotFoundException
 from airflow.hooks.base import BaseHook
-from airflow.models.connection import Connection
 from airflow.providers.amazon.aws.utils.connection_wrapper import AwsConnectionWrapper
+from airflow.providers.amazon.aws.waiters.base_waiter import BaseBotoWaiter
+from airflow.providers_manager import ProvidersManager
+from airflow.utils.helpers import exactly_one
 from airflow.utils.log.logging_mixin import LoggingMixin
 from airflow.utils.log.secrets_masker import mask_secret
 
 BaseAwsConnection = TypeVar("BaseAwsConnection", bound=Union[boto3.client, boto3.resource])
 
 
+if TYPE_CHECKING:
+    from airflow.models.connection import Connection  # Avoid circular imports.
+
+
 class BaseSessionFactory(LoggingMixin):
     """
     Base AWS Session Factory class to handle boto3 session creation.
@@ -122,17 +135,29 @@ class BaseSessionFactory(LoggingMixin):
             return boto3.session.Session(region_name=self.region_name)
         elif not self.role_arn:
             return self.basic_session
-        return self._create_session_with_assume_role(session_kwargs=self.conn.session_kwargs)
+
+        # Values stored in ``AwsConnectionWrapper.session_kwargs`` are intended to be used only
+        # to create the initial boto3 session.
+        # If the user wants to use the 'assume_role' mechanism then only the 'region_name' needs to be
+        # provided, otherwise other parameters might conflict with the base botocore session.
+        # Unfortunately it is not a part of public boto3 API, see source of boto3.session.Session:
+        # https://boto3.amazonaws.com/v1/documentation/api/latest/_modules/boto3/session.html#Session
+        # If we provide 'aws_access_key_id' or 'aws_secret_access_key' or 'aws_session_token'
+        # as part of session kwargs it will use them instead of assumed credentials.
+        assume_session_kwargs = {}
+        if self.conn.region_name:
+            assume_session_kwargs["region_name"] = self.conn.region_name
+        return self._create_session_with_assume_role(session_kwargs=assume_session_kwargs)
 
     def _create_basic_session(self, session_kwargs: dict[str, Any]) -> boto3.session.Session:
         return boto3.session.Session(**session_kwargs)
 
     def _create_session_with_assume_role(self, session_kwargs: dict[str, Any]) -> boto3.session.Session:
-        if self.conn.assume_role_method == 'assume_role_with_web_identity':
+        if self.conn.assume_role_method == "assume_role_with_web_identity":
             # Deferred credentials have no initial credentials
             credential_fetcher = self._get_web_identity_credential_fetcher()
             credentials = botocore.credentials.DeferredRefreshableCredentials(
-                method='assume-role-with-web-identity',
+                method="assume-role-with-web-identity",
                 refresh_using=credential_fetcher.fetch_credentials,
                 time_fetcher=lambda: datetime.datetime.now(tz=tzlocal()),
             )
@@ -152,25 +177,25 @@ class BaseSessionFactory(LoggingMixin):
         return boto3.session.Session(botocore_session=session, **session_kwargs)
 
     def _refresh_credentials(self) -> dict[str, Any]:
-        self.log.debug('Refreshing credentials')
+        self.log.debug("Refreshing credentials")
         assume_role_method = self.conn.assume_role_method
-        if assume_role_method not in ('assume_role', 'assume_role_with_saml'):
-            raise NotImplementedError(f'assume_role_method={assume_role_method} not expected')
+        if assume_role_method not in ("assume_role", "assume_role_with_saml"):
+            raise NotImplementedError(f"assume_role_method={assume_role_method} not expected")
 
         sts_client = self.basic_session.client("sts", config=self.config)
 
-        if assume_role_method == 'assume_role':
+        if assume_role_method == "assume_role":
             sts_response = self._assume_role(sts_client=sts_client)
         else:
             sts_response = self._assume_role_with_saml(sts_client=sts_client)
 
-        sts_response_http_status = sts_response['ResponseMetadata']['HTTPStatusCode']
+        sts_response_http_status = sts_response["ResponseMetadata"]["HTTPStatusCode"]
         if sts_response_http_status != 200:
-            raise RuntimeError(f'sts_response_http_status={sts_response_http_status}')
+            raise RuntimeError(f"sts_response_http_status={sts_response_http_status}")
 
-        credentials = sts_response['Credentials']
-        expiry_time = credentials.get('Expiration').isoformat()
-        self.log.debug('New credentials expiry_time: %s', expiry_time)
+        credentials = sts_response["Credentials"]
+        expiry_time = credentials.get("Expiration").isoformat()
+        self.log.debug("New credentials expiry_time: %s", expiry_time)
         credentials = {
             "access_key": credentials.get("AccessKeyId"),
             "secret_key": credentials.get("SecretAccessKey"),
@@ -188,15 +213,15 @@ class BaseSessionFactory(LoggingMixin):
         return sts_client.assume_role(**kw)
 
     def _assume_role_with_saml(self, sts_client: boto3.client) -> dict[str, Any]:
-        saml_config = self.extra_config['assume_role_with_saml']
-        principal_arn = saml_config['principal_arn']
+        saml_config = self.extra_config["assume_role_with_saml"]
+        principal_arn = saml_config["principal_arn"]
 
-        idp_auth_method = saml_config['idp_auth_method']
-        if idp_auth_method == 'http_spegno_auth':
+        idp_auth_method = saml_config["idp_auth_method"]
+        if idp_auth_method == "http_spegno_auth":
             saml_assertion = self._fetch_saml_assertion_using_http_spegno_auth(saml_config)
         else:
             raise NotImplementedError(
-                f'idp_auth_method={idp_auth_method} in Connection {self.conn.conn_id} Extra.'
+                f"idp_auth_method={idp_auth_method} in Connection {self.conn.conn_id} Extra."
                 'Currently only "http_spegno_auth" is supported, and must be specified.'
             )
 
@@ -245,32 +270,32 @@ class BaseSessionFactory(LoggingMixin):
         from lxml import etree
 
         auth = requests_gssapi.HTTPSPNEGOAuth()
-        if 'mutual_authentication' in saml_config:
-            mutual_auth = saml_config['mutual_authentication']
-            if mutual_auth == 'REQUIRED':
+        if "mutual_authentication" in saml_config:
+            mutual_auth = saml_config["mutual_authentication"]
+            if mutual_auth == "REQUIRED":
                 auth = requests_gssapi.HTTPSPNEGOAuth(requests_gssapi.REQUIRED)
-            elif mutual_auth == 'OPTIONAL':
+            elif mutual_auth == "OPTIONAL":
                 auth = requests_gssapi.HTTPSPNEGOAuth(requests_gssapi.OPTIONAL)
-            elif mutual_auth == 'DISABLED':
+            elif mutual_auth == "DISABLED":
                 auth = requests_gssapi.HTTPSPNEGOAuth(requests_gssapi.DISABLED)
             else:
                 raise NotImplementedError(
-                    f'mutual_authentication={mutual_auth} in Connection {self.conn.conn_id} Extra.'
+                    f"mutual_authentication={mutual_auth} in Connection {self.conn.conn_id} Extra."
                     'Currently "REQUIRED", "OPTIONAL" and "DISABLED" are supported.'
-                    '(Exclude this setting will default to HTTPSPNEGOAuth() ).'
+                    "(Exclude this setting will default to HTTPSPNEGOAuth() )."
                 )
         # Query the IDP
         idp_response = self._get_idp_response(saml_config, auth=auth)
         # Assist with debugging. Note: contains sensitive info!
-        xpath = saml_config['saml_response_xpath']
-        log_idp_response = 'log_idp_response' in saml_config and saml_config['log_idp_response']
+        xpath = saml_config["saml_response_xpath"]
+        log_idp_response = "log_idp_response" in saml_config and saml_config["log_idp_response"]
         if log_idp_response:
             self.log.warning(
-                'The IDP response contains sensitive information, but log_idp_response is ON (%s).',
+                "The IDP response contains sensitive information, but log_idp_response is ON (%s).",
                 log_idp_response,
             )
-            self.log.debug('idp_response.content= %s', idp_response.content)
-            self.log.debug('xpath= %s', xpath)
+            self.log.debug("idp_response.content= %s", idp_response.content)
+            self.log.debug("xpath= %s", xpath)
         # Extract SAML Assertion from the returned HTML / XML
         xml = etree.fromstring(idp_response.content)
         saml_assertion = xml.xpath(xpath)
@@ -278,7 +303,7 @@ class BaseSessionFactory(LoggingMixin):
             if len(saml_assertion) == 1:
                 saml_assertion = saml_assertion[0]
         if not saml_assertion:
-            raise ValueError('Invalid SAML Assertion')
+            raise ValueError("Invalid SAML Assertion")
         return saml_assertion
 
     def _get_web_identity_credential_fetcher(
@@ -286,8 +311,8 @@ class BaseSessionFactory(LoggingMixin):
     ) -> botocore.credentials.AssumeRoleWithWebIdentityCredentialFetcher:
         base_session = self.basic_session._session or botocore.session.get_session()
         client_creator = base_session.create_client
-        federation = self.extra_config.get('assume_role_with_web_identity_federation')
-        if federation == 'google':
+        federation = self.extra_config.get("assume_role_with_web_identity_federation")
+        if federation == "google":
             web_identity_token_loader = self._get_google_identity_token_loader()
         else:
             raise AirflowException(
@@ -307,7 +332,7 @@ class BaseSessionFactory(LoggingMixin):
             get_default_id_token_credentials,
         )
 
-        audience = self.extra_config.get('assume_role_with_web_identity_federation_audience')
+        audience = self.extra_config.get("assume_role_with_web_identity_federation_audience")
 
         google_id_token_credentials = get_default_id_token_credentials(target_audience=audience)
 
@@ -320,7 +345,7 @@ class BaseSessionFactory(LoggingMixin):
         return web_identity_token_loader
 
     def _strip_invalid_session_name_characters(self, role_session_name: str) -> str:
-        return slugify(role_session_name, regex_pattern=r'[^\w+=,.@-]+')
+        return slugify(role_session_name, regex_pattern=r"[^\w+=,.@-]+")
 
     def _get_region_name(self) -> str | None:
         warnings.warn(
@@ -370,10 +395,10 @@ class AwsGenericHook(BaseHook, Generic[BaseAwsConnection]):
         https://botocore.amazonaws.com/v1/documentation/api/latest/reference/config.html
     """
 
-    conn_name_attr = 'aws_conn_id'
-    default_conn_name = 'aws_default'
-    conn_type = 'aws'
-    hook_name = 'Amazon Web Services'
+    conn_name_attr = "aws_conn_id"
+    default_conn_name = "aws_default"
+    conn_type = "aws"
+    hook_name = "Amazon Web Services"
 
     def __init__(
         self,
@@ -393,6 +418,92 @@ class AwsGenericHook(BaseHook, Generic[BaseAwsConnection]):
         self._config = config
         self._verify = verify
 
+    @classmethod
+    def _get_provider_version(cls) -> str:
+        """Checks the Providers Manager for the package version."""
+        try:
+            manager = ProvidersManager()
+            hook = manager.hooks[cls.conn_type]
+            if not hook:
+                # This gets caught immediately, but without it MyPy complains
+                # Item "None" of "Optional[HookInfo]" has no attribute "package_name"
+                # on the following line and static checks fail.
+                raise ValueError(f"Hook info for {cls.conn_type} not found in the Provider Manager.")
+            provider = manager.providers[hook.package_name]
+            return provider.version
+        except Exception:
+            # Under no condition should an error here ever cause an issue for the user.
+            return "Unknown"
+
+    @staticmethod
+    def _find_class_name(target_function_name: str) -> str:
+        """
+        Given a frame off the stack, return the name of the class which made the call.
+        Note: This method may raise a ValueError or an IndexError, but the calling
+        method is catching and handling those.
+        """
+        stack = inspect.stack()
+        # Find the index of the most recent frame which called the provided function name.
+        target_frame_index = [frame.function for frame in stack].index(target_function_name)
+        # Pull that frame off the stack.
+        target_frame = stack[target_frame_index][0]
+        # Get the local variables for that frame.
+        frame_variables = target_frame.f_locals["self"]
+        # Get the class object for that frame.
+        frame_class_object = frame_variables.__class__
+        # Return the name of the class object.
+        return frame_class_object.__name__
+
+    def _get_caller(self, target_function_name: str = "execute") -> str:
+        """Given a function name, walk the stack and return the name of the class which called it last."""
+        try:
+            caller = self._find_class_name(target_function_name)
+            if caller == "BaseSensorOperator":
+                # If the result is a BaseSensorOperator, then look for whatever last called "poke".
+                return self._get_caller("poke")
+            return caller
+        except Exception:
+            # Under no condition should an error here ever cause an issue for the user.
+            return "Unknown"
+
+    @staticmethod
+    def _generate_dag_key() -> str:
+        """
+        The Object Identifier (OID) namespace is used to salt the dag_id value.
+        That salted value is used to generate a SHA-1 hash which, by definition,
+        can not (reasonably) be reversed.  No personal data can be inferred or
+        extracted from the resulting UUID.
+        """
+        try:
+            dag_id = os.environ["AIRFLOW_CTX_DAG_ID"]
+            return str(uuid.uuid5(uuid.NAMESPACE_OID, dag_id))
+        except Exception:
+            # Under no condition should an error here ever cause an issue for the user.
+            return "00000000-0000-0000-0000-000000000000"
+
+    @staticmethod
+    def _get_airflow_version() -> str:
+        """Fetch and return the current Airflow version."""
+        try:
+            # This can be a circular import under specific configurations.
+            # Importing locally to either avoid or catch it if it does happen.
+            from airflow import __version__ as airflow_version
+
+            return airflow_version
+        except Exception:
+            # Under no condition should an error here ever cause an issue for the user.
+            return "Unknown"
+
+    def _generate_user_agent_extra_field(self, existing_user_agent_extra: str) -> str:
+        user_agent_extra_values = [
+            f"Airflow/{self._get_airflow_version()}",
+            f"AmPP/{self._get_provider_version()}",
+            f"Caller/{self._get_caller()}",
+            f"DagRunKey/{self._generate_dag_key()}",
+            existing_user_agent_extra or "",
+        ]
+        return " ".join(user_agent_extra_values).strip()
+
     @cached_property
     def conn_config(self) -> AwsConnectionWrapper:
         """Get the Airflow Connection object and wrap it in helper (cached)."""
@@ -420,9 +531,9 @@ class AwsGenericHook(BaseHook, Generic[BaseAwsConnection]):
         return self.conn_config.region_name
 
     @property
-    def config(self) -> Config | None:
+    def config(self) -> Config:
         """Configuration for botocore client read-only property."""
-        return self.conn_config.botocore_config
+        return self.conn_config.botocore_config or botocore.config.Config()
 
     @property
     def verify(self) -> bool | str | None:
@@ -435,6 +546,23 @@ class AwsGenericHook(BaseHook, Generic[BaseAwsConnection]):
             conn=self.conn_config, region_name=region_name, config=self.config
         ).create_session()
 
+    def _get_config(self, config: Config | None = None) -> Config:
+        """
+        No AWS Operators use the config argument to this method.
+        Keep backward compatibility with other users who might use it
+        """
+        if config is None:
+            config = deepcopy(self.config)
+
+        # ignore[union-attr] is required for this block to appease MyPy
+        # because the user_agent_extra field is generated at runtime.
+        user_agent_config = Config(
+            user_agent_extra=self._generate_user_agent_extra_field(
+                existing_user_agent_extra=config.user_agent_extra  # type: ignore[union-attr]
+            )
+        )
+        return config.merge(user_agent_config)  # type: ignore[union-attr]
+
     def get_client_type(
         self,
         region_name: str | None = None,
@@ -442,15 +570,12 @@ class AwsGenericHook(BaseHook, Generic[BaseAwsConnection]):
     ) -> boto3.client:
         """Get the underlying boto3 client using boto3 session"""
         client_type = self.client_type
-
-        # No AWS Operators use the config argument to this method.
-        # Keep backward compatibility with other users who might use it
-        if config is None:
-            config = self.config
-
         session = self.get_session(region_name=region_name)
         return session.client(
-            client_type, endpoint_url=self.conn_config.endpoint_url, config=config, verify=self.verify
+            client_type,
+            endpoint_url=self.conn_config.endpoint_url,
+            config=self._get_config(config),
+            verify=self.verify,
         )
 
     def get_resource_type(
@@ -460,15 +585,12 @@ class AwsGenericHook(BaseHook, Generic[BaseAwsConnection]):
     ) -> boto3.resource:
         """Get the underlying boto3 resource using boto3 session"""
         resource_type = self.resource_type
-
-        # No AWS Operators use the config argument to this method.
-        # Keep backward compatibility with other users who might use it
-        if config is None:
-            config = self.config
-
         session = self.get_session(region_name=region_name)
         return session.resource(
-            resource_type, endpoint_url=self.conn_config.endpoint_url, config=config, verify=self.verify
+            resource_type,
+            endpoint_url=self.conn_config.endpoint_url,
+            config=self._get_config(config),
+            verify=self.verify,
         )
 
     @cached_property
@@ -477,9 +599,8 @@ class AwsGenericHook(BaseHook, Generic[BaseAwsConnection]):
         Get the underlying boto3 client/resource (cached)
 
         :return: boto3.client or boto3.resource
-        :rtype: Union[boto3.client, boto3.resource]
         """
-        if not ((not self.client_type) ^ (not self.resource_type)):
+        if not exactly_one(self.client_type, self.resource_type):
             raise ValueError(
                 f"Either client_type={self.client_type!r} or "
                 f"resource_type={self.resource_type!r} must be provided, not both."
@@ -515,7 +636,6 @@ class AwsGenericHook(BaseHook, Generic[BaseAwsConnection]):
         with subclasses that rely on a super().get_conn() method.
 
         :return: boto3.client or boto3.resource
-        :rtype: Union[boto3.client, boto3.resource]
         """
         # Compat shim
         return self.conn
@@ -550,7 +670,7 @@ class AwsGenericHook(BaseHook, Generic[BaseAwsConnection]):
         else:
             session = self.get_session(region_name=region_name)
             _client = session.client(
-                'iam', endpoint_url=self.conn_config.endpoint_url, config=self.config, verify=self.verify
+                "iam", endpoint_url=self.conn_config.endpoint_url, config=self.config, verify=self.verify
             )
             return _client.get_role(RoleName=role)["Role"]["Arn"]
 
@@ -564,21 +684,21 @@ class AwsGenericHook(BaseHook, Generic[BaseAwsConnection]):
         def retry_decorator(fun: Callable):
             @wraps(fun)
             def decorator_f(self, *args, **kwargs):
-                retry_args = getattr(self, 'retry_args', None)
+                retry_args = getattr(self, "retry_args", None)
                 if retry_args is None:
                     return fun(self, *args, **kwargs)
-                multiplier = retry_args.get('multiplier', 1)
-                min_limit = retry_args.get('min', 1)
-                max_limit = retry_args.get('max', 1)
-                stop_after_delay = retry_args.get('stop_after_delay', 10)
+                multiplier = retry_args.get("multiplier", 1)
+                min_limit = retry_args.get("min", 1)
+                max_limit = retry_args.get("max", 1)
+                stop_after_delay = retry_args.get("stop_after_delay", 10)
                 tenacity_before_logger = tenacity.before_log(self.log, logging.INFO) if self.log else None
                 tenacity_after_logger = tenacity.after_log(self.log, logging.INFO) if self.log else None
                 default_kwargs = {
-                    'wait': tenacity.wait_exponential(multiplier=multiplier, max=max_limit, min=min_limit),
-                    'retry': tenacity.retry_if_exception(should_retry),
-                    'stop': tenacity.stop_after_delay(stop_after_delay),
-                    'before': tenacity_before_logger,
-                    'after': tenacity_after_logger,
+                    "wait": tenacity.wait_exponential(multiplier=multiplier, max=max_limit, min=min_limit),
+                    "retry": tenacity.retry_if_exception(should_retry),
+                    "stop": tenacity.stop_after_delay(stop_after_delay),
+                    "before": tenacity_before_logger,
+                    "after": tenacity_after_logger,
                 }
                 return tenacity.retry(**default_kwargs)(fun)(self, *args, **kwargs)
 
@@ -632,21 +752,59 @@ class AwsGenericHook(BaseHook, Generic[BaseAwsConnection]):
         .. seealso::
             https://docs.aws.amazon.com/STS/latest/APIReference/API_GetCallerIdentity.html
         """
-        orig_client_type, self.client_type = self.client_type, 'sts'
         try:
-            res = self.get_client_type().get_caller_identity()
-            metadata = res.pop("ResponseMetadata", {})
-            if metadata.get("HTTPStatusCode") == 200:
-                return True, json.dumps(res)
-            else:
+            session = self.get_session()
+            conn_info = session.client("sts").get_caller_identity()
+            metadata = conn_info.pop("ResponseMetadata", {})
+            if metadata.get("HTTPStatusCode") != 200:
                 try:
                     return False, json.dumps(metadata)
                 except TypeError:
                     return False, str(metadata)
+            conn_info["credentials_method"] = session.get_credentials().method
+            conn_info["region_name"] = session.region_name
+            return True, ", ".join(f"{k}={v!r}" for k, v in conn_info.items())
+
         except Exception as e:
-            return False, str(e)
-        finally:
-            self.client_type = orig_client_type
+            return False, str(f"{type(e).__name__!r} error occurred while testing connection: {e}")
+
+    @cached_property
+    def waiter_path(self) -> PathLike[str] | None:
+        path = Path(__file__).parents[1].joinpath(f"waiters/{self.client_type}.json").resolve()
+        return path if path.exists() else None
+
+    def get_waiter(self, waiter_name: str) -> Waiter:
+        """
+        First checks if there is a custom waiter with the provided waiter_name and
+        uses that if it exists, otherwise it will check the service client for a
+        waiter that matches the name and pass that through.
+
+        :param waiter_name: The name of the waiter.  The name should exactly match the
+            name of the key in the waiter model file (typically this is CamelCase).
+        """
+        if self.waiter_path and (waiter_name in self._list_custom_waiters()):
+            # Technically if waiter_name is in custom_waiters then self.waiter_path must
+            # exist but MyPy doesn't like the fact that self.waiter_path could be None.
+            with open(self.waiter_path) as config_file:
+                config = json.load(config_file)
+                return BaseBotoWaiter(client=self.conn, model_config=config).waiter(waiter_name)
+        # If there is no custom waiter found for the provided name,
+        # then try checking the service's official waiters.
+        return self.conn.get_waiter(waiter_name)
+
+    def list_waiters(self) -> list[str]:
+        """Returns a list containing the names of all waiters for the service, official and custom."""
+        return [*self._list_official_waiters(), *self._list_custom_waiters()]
+
+    def _list_official_waiters(self) -> list[str]:
+        return self.conn.waiter_names
+
+    def _list_custom_waiters(self) -> list[str]:
+        if not self.waiter_path:
+            return []
+        with open(self.waiter_path) as config_file:
+            model_config = json.load(config_file)
+            return WaiterModel(model_config).waiter_names
 
 
 class AwsBaseHook(AwsGenericHook[Union[boto3.client, boto3.resource]]):
