@@ -25,16 +25,9 @@ An Airflow operator for AWS Batch services
 """
 from __future__ import annotations
 
-import sys
-from typing import TYPE_CHECKING, Any, Optional, Sequence
-
-from airflow.providers.amazon.aws.utils import trim_none_values
-
-if sys.version_info >= (3, 8):
-    from functools import cached_property
-else:
-    from cached_property import cached_property
+from typing import TYPE_CHECKING, Any, Sequence
 
+from airflow.compat.functools import cached_property
 from airflow.exceptions import AirflowException
 from airflow.models import BaseOperator
 from airflow.providers.amazon.aws.hooks.batch_client import BatchClientHook
@@ -44,6 +37,7 @@ from airflow.providers.amazon.aws.links.batch import (
     BatchJobQueueLink,
 )
 from airflow.providers.amazon.aws.links.logs import CloudWatchEventsLink
+from airflow.providers.amazon.aws.utils import trim_none_values
 
 if TYPE_CHECKING:
     from airflow.utils.context import Context
@@ -100,13 +94,18 @@ class BatchOperator(BaseOperator):
     """
 
     ui_color = "#c3dae0"
-    arn = None  # type: Optional[str]
+    arn: str | None = None
     template_fields: Sequence[str] = (
+        "job_id",
         "job_name",
-        "job_queue",
         "job_definition",
+        "job_queue",
         "overrides",
+        "array_properties",
         "parameters",
+        "waiters",
+        "tags",
+        "wait_for_completion",
     )
     template_fields_renderers = {"overrides": "json", "parameters": "json"}
 
@@ -227,7 +226,7 @@ class BatchOperator(BaseOperator):
         :raises: AirflowException
         """
         if not self.job_id:
-            raise AirflowException('AWS Batch job - job_id was not found')
+            raise AirflowException("AWS Batch job - job_id was not found")
 
         try:
             job_desc = self.hook.get_job_description(self.job_id)
@@ -365,14 +364,14 @@ class BatchCreateComputeEnvironmentOperator(BaseOperator):
     def execute(self, context: Context):
         """Create an AWS batch compute environment"""
         kwargs: dict[str, Any] = {
-            'computeEnvironmentName': self.compute_environment_name,
-            'type': self.environment_type,
-            'state': self.state,
-            'unmanagedvCpus': self.unmanaged_v_cpus,
-            'computeResources': self.compute_resources,
-            'serviceRole': self.service_role,
-            'tags': self.tags,
+            "computeEnvironmentName": self.compute_environment_name,
+            "type": self.environment_type,
+            "state": self.state,
+            "unmanagedvCpus": self.unmanaged_v_cpus,
+            "computeResources": self.compute_resources,
+            "serviceRole": self.service_role,
+            "tags": self.tags,
         }
         self.hook.client.create_compute_environment(**trim_none_values(kwargs))
 
-        self.log.info('AWS Batch compute environment created successfully')
+        self.log.info("AWS Batch compute environment created successfully")
