@@ -22,8 +22,13 @@ from collections import namedtuple
 from tempfile import NamedTemporaryFile
 from typing import TYPE_CHECKING, Iterable, Mapping, Sequence
 
-import numpy as np
-import pandas as pd
+try:
+    import numpy as np
+    import pandas as pd
+except ImportError as e:
+    from airflow.exceptions import AirflowOptionalProviderFeatureException
+
+    raise AirflowOptionalProviderFeatureException(e)
 from typing_extensions import Literal
 
 from airflow.exceptions import AirflowException
@@ -44,12 +49,12 @@ class FILE_FORMAT(enum.Enum):
     PARQUET = enum.auto()
 
 
-FileOptions = namedtuple('FileOptions', ['mode', 'suffix', 'function'])
+FileOptions = namedtuple("FileOptions", ["mode", "suffix", "function"])
 
 FILE_OPTIONS_MAP = {
-    FILE_FORMAT.CSV: FileOptions('r+', '.csv', 'to_csv'),
-    FILE_FORMAT.JSON: FileOptions('r+', '.json', 'to_json'),
-    FILE_FORMAT.PARQUET: FileOptions('rb+', '.parquet', 'to_parquet'),
+    FILE_FORMAT.CSV: FileOptions("r+", ".csv", "to_csv"),
+    FILE_FORMAT.JSON: FileOptions("r+", ".json", "to_json"),
+    FILE_FORMAT.PARQUET: FileOptions("rb+", ".parquet", "to_parquet"),
 }
 
 
@@ -83,11 +88,12 @@ class SqlToS3Operator(BaseOperator):
     """
 
     template_fields: Sequence[str] = (
-        's3_bucket',
-        's3_key',
-        'query',
+        "s3_bucket",
+        "s3_key",
+        "query",
+        "sql_conn_id",
     )
-    template_ext: Sequence[str] = ('.sql',)
+    template_ext: Sequence[str] = (".sql",)
     template_fields_renderers = {
         "query": "sql",
         "pd_kwargs": "json",
@@ -102,9 +108,9 @@ class SqlToS3Operator(BaseOperator):
         sql_conn_id: str,
         parameters: None | Mapping | Iterable = None,
         replace: bool = False,
-        aws_conn_id: str = 'aws_default',
+        aws_conn_id: str = "aws_default",
         verify: bool | str | None = None,
-        file_format: Literal['csv', 'json', 'parquet'] = 'csv',
+        file_format: Literal["csv", "json", "parquet"] = "csv",
         pd_kwargs: dict | None = None,
         **kwargs,
     ) -> None:
@@ -120,7 +126,7 @@ class SqlToS3Operator(BaseOperator):
         self.parameters = parameters
 
         if "path_or_buf" in self.pd_kwargs:
-            raise AirflowException('The argument path_or_buf is not allowed, please remove it')
+            raise AirflowException("The argument path_or_buf is not allowed, please remove it")
 
         try:
             self.file_format = FILE_FORMAT[file_format.upper()]
@@ -128,14 +134,14 @@ class SqlToS3Operator(BaseOperator):
             raise AirflowException(f"The argument file_format doesn't support {file_format} value.")
 
     @staticmethod
-    def _fix_dtypes(df: pd.DataFrame) -> None:
+    def _fix_dtypes(df: pd.DataFrame, file_format: FILE_FORMAT) -> None:
         """
         Mutate DataFrame to set dtypes for float columns containing NaN values.
         Set dtype of object to str to allow for downstream transformations.
         """
         for col in df:
 
-            if df[col].dtype.name == 'object':
+            if df[col].dtype.name == "object" and file_format == "parquet":
                 # if the type wasn't identified or converted, change it to a string so if can still be
                 # processed.
                 df[col] = df[col].astype(str)
@@ -158,7 +164,7 @@ class SqlToS3Operator(BaseOperator):
         data_df = sql_hook.get_pandas_df(sql=self.query, parameters=self.parameters)
         self.log.info("Data from SQL obtained")
 
-        self._fix_dtypes(data_df)
+        self._fix_dtypes(data_df, self.file_format)
         file_options = FILE_OPTIONS_MAP[self.file_format]
 
         with NamedTemporaryFile(mode=file_options.mode, suffix=file_options.suffix) as tmp_file:
@@ -175,7 +181,7 @@ class SqlToS3Operator(BaseOperator):
         self.log.debug("Get connection for %s", self.sql_conn_id)
         conn = BaseHook.get_connection(self.sql_conn_id)
         hook = conn.get_hook()
-        if not callable(getattr(hook, 'get_pandas_df', None)):
+        if not callable(getattr(hook, "get_pandas_df", None)):
             raise AirflowException(
                 "This hook is not supported. The hook class must have get_pandas_df method."
             )
